#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2014 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core.Views;
using HeuristicLab.MainForm;
using HeuristicLab.PluginInfrastructure;
using System;
using System.Windows.Forms;

namespace HeuristicLab.Core.Networks.Views {
  [View("PortParameter View")]
  [Content(typeof(PortParameter<>), true)]
  [Content(typeof(IPortParameter<>), false)]
  [Content(typeof(IPortParameter), false)]
  public partial class PortParameterView : NetworkItemView {
    protected TypeSelectorDialog typeSelectorDialog;

    public new IPortParameter Content {
      get { return (IPortParameter)base.Content; }
      set { base.Content = value; }
    }

    public PortParameterView() {
      InitializeComponent();
    }

    protected override void Dispose(bool disposing) {
      if (disposing) {
        if (typeSelectorDialog != null) typeSelectorDialog.Dispose();
        if (components != null) components.Dispose();
      }
      base.Dispose(disposing);
    }

    protected override void DeregisterContentEvents() {
      Content.TypeChanged -= Content_TypeChanged;
      Content.DefaultValueChanged -= Content_DefaultValueChanged;
      base.DeregisterContentEvents();
    }

    protected override void RegisterContentEvents() {
      base.RegisterContentEvents();
      Content.TypeChanged += Content_TypeChanged;
      Content.DefaultValueChanged += Content_DefaultValueChanged;
    }

    protected override void OnContentChanged() {
      base.OnContentChanged();
      if (Content == null) {
        dataTypeTextBox.Text = string.Empty;
        inputTypeCheckBox.Checked = false;
        outputTypeCheckBox.Checked = false;
        defaultValueViewHost.Content = null;
      } else {
        dataTypeTextBox.Text = Content.DataType.GetPrettyName();
        inputTypeCheckBox.Checked = Content.Type.HasFlag(PortParameterType.Input);
        outputTypeCheckBox.Checked = Content.Type.HasFlag(PortParameterType.Output);
        defaultValueViewHost.ViewType = null;
        defaultValueViewHost.Content = Content.DefaultValue;
      }
    }

    protected override void SetEnabledStateOfControls() {
      base.SetEnabledStateOfControls();
      dataTypeTextBox.Enabled = Content != null;
      inputTypeCheckBox.Enabled = Content != null && !ReadOnly;
      outputTypeCheckBox.Enabled = Content != null && !ReadOnly;
      defaultValueGroupBox.Enabled = Content != null;
      setDefaultValueButton.Enabled = Content != null && !ReadOnly;
      clearDefaultValueButton.Enabled = Content != null && Content.DefaultValue != null && !ReadOnly;
    }
    protected virtual void Content_TypeChanged(object sender, EventArgs e) {
      if (InvokeRequired)
        Invoke(new EventHandler(Content_TypeChanged), sender, e);
      else {
        inputTypeCheckBox.Checked = Content.Type.HasFlag(PortParameterType.Input);
        outputTypeCheckBox.Checked = Content.Type.HasFlag(PortParameterType.Output);
      }
    }
    protected virtual void Content_DefaultValueChanged(object sender, System.EventArgs e) {
      if (InvokeRequired)
        Invoke(new EventHandler(Content_DefaultValueChanged), sender, e);
      else {
        clearDefaultValueButton.Enabled = Content.DefaultValue != null;
        defaultValueViewHost.ViewType = null;
        defaultValueViewHost.Content = Content.DefaultValue;
      }
    }

    protected virtual void inputTypeCheckBox_CheckedChanged(object sender, EventArgs e) {
      Content.Type = inputTypeCheckBox.Checked ? Content.Type | PortParameterType.Input : Content.Type & ~PortParameterType.Input;
    }
    protected virtual void outputTypeCheckBox_CheckedChanged(object sender, EventArgs e) {
      Content.Type = outputTypeCheckBox.Checked ? Content.Type | PortParameterType.Output : Content.Type & ~PortParameterType.Output;
    }

    protected virtual void setDefaultValueButton_Click(object sender, EventArgs e) {
      if (typeSelectorDialog == null) {
        typeSelectorDialog = new TypeSelectorDialog();
        typeSelectorDialog.Caption = "Select Default Value";
      }
      typeSelectorDialog.TypeSelector.Configure(Content.DataType, false, true);
      if (typeSelectorDialog.ShowDialog(this) == DialogResult.OK) {
        try {
          Content.DefaultValue = (IItem)typeSelectorDialog.TypeSelector.CreateInstanceOfSelectedType();
        }
        catch (Exception ex) {
          ErrorHandling.ShowErrorDialog(this, ex);
        }
      }
    }
    protected virtual void clearDefaultValueButton_Click(object sender, EventArgs e) {
      Content.DefaultValue = null;
    }

    protected virtual void defaultValuePanel_DragEnterOver(object sender, DragEventArgs e) {
      e.Effect = DragDropEffects.None;
      if (!ReadOnly && (e.Data.GetData(HeuristicLab.Common.Constants.DragDropDataFormat) is IItem)) {
        if ((e.KeyState & 32) == 32) e.Effect = DragDropEffects.Link;  // ALT key
        else if ((e.KeyState & 4) == 4) e.Effect = DragDropEffects.Move;  // SHIFT key
        else if (e.AllowedEffect.HasFlag(DragDropEffects.Copy)) e.Effect = DragDropEffects.Copy;
        else if (e.AllowedEffect.HasFlag(DragDropEffects.Move)) e.Effect = DragDropEffects.Move;
        else if (e.AllowedEffect.HasFlag(DragDropEffects.Link)) e.Effect = DragDropEffects.Link;
      }
    }
    protected virtual void defaultValuePanel_DragDrop(object sender, DragEventArgs e) {
      if (e.Effect != DragDropEffects.None) {
        IItem item = e.Data.GetData(HeuristicLab.Common.Constants.DragDropDataFormat) as IItem;
        if (e.Effect.HasFlag(DragDropEffects.Copy)) item = (IItem)item.Clone();
        Content.DefaultValue = item;
      }
    }
  }
}
